package com.fusi24.entryPermitScanner.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.annimon.stream.Collectors;
import com.annimon.stream.Stream;
import com.fusi24.entryPermitScanner.R;
import com.fusi24.entryPermitScanner.data.entity.DataGroup;
import com.fusi24.entryPermitScanner.data.entity.workingpermit.DataResultPermit;

import java.util.ArrayList;
import java.util.List;


public class ProblemWorkingAdapter extends RecyclerView.Adapter<ProblemWorkingAdapter.ViewHolder>{

    private Context context;
    private List<DataResultPermit> resultPermits;
    private List<DataGroup> groupList;
    private RecyclerView.RecycledViewPool viewPool = new RecyclerView.RecycledViewPool();

    public ProblemWorkingAdapter(Context context) {
        this.context = context;
        this.resultPermits = new ArrayList<>();
        this.groupList = new ArrayList<>();
    }

    public List<DataResultPermit> getResultPermits() {
        return resultPermits;
    }

    public void setResultPermits(List<DataResultPermit> resultPermits) {
        this.resultPermits = resultPermits;
    }

    public void updateResultPermits(List<DataResultPermit> resultPermits){
        setResultPermits(resultPermits);
    }

    static class ViewHolder extends RecyclerView.ViewHolder {

        private com.fusi24.entryPermitScanner.databinding.RowGroupBinding binding;

        ViewHolder(com.fusi24.entryPermitScanner.databinding.RowGroupBinding itemBinding) {
            super(itemBinding.getRoot());
            this.binding = itemBinding;
        }
    }


    @NonNull
    @Override
    public ProblemWorkingAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        com.fusi24.entryPermitScanner.databinding.RowGroupBinding itemBinding = com.fusi24.entryPermitScanner.databinding.
                RowGroupBinding.inflate(LayoutInflater.from(context), parent, false);
        return new ViewHolder(itemBinding);
    }

    @Override
    public void onBindViewHolder(@NonNull ProblemWorkingAdapter.ViewHolder holder, int position) {

        DataResultPermit permit = resultPermits.get(position);
        holder.binding.tvGroupName.setText(permit.getWorkingType().getName());

        if (permit.getProblems().size() == 0){
            holder.binding.tvMessage.setVisibility(View.VISIBLE);
            holder.binding.tvMessage.setText(permit.getMessage());
        }

        groupList = Stream.of(permit.getProblems())
                .map(v -> v.getDataDocumentType().getGroup())
                .distinctBy(DataGroup::getId)
                .collect(Collectors.toList());

        // Create layout manager with initial prefetch item count
        LinearLayoutManager layoutManager = new LinearLayoutManager(
                holder.binding.rvSubGroup.getContext(),
                LinearLayoutManager.VERTICAL, false);

        layoutManager.setInitialPrefetchItemCount(groupList.size());

        //Create Sub Item View Adapter
        ProblemGroupAdapter groupAdapter = new ProblemGroupAdapter(context);
        holder.binding.rvSubGroup.setLayoutManager(layoutManager);
        holder.binding.rvSubGroup.setAdapter(groupAdapter);
        holder.binding.rvSubGroup.setRecycledViewPool(viewPool);

        groupAdapter.updateGroupList(groupList);
        groupAdapter.updateProblemList(permit.getProblems());
        groupAdapter.notifyDataSetChanged();

        holder.binding.ivIconGroup.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_person_hand_right_up));
        holder.binding.rlGroup.setOnClickListener(v -> {
            // Set Layout
            if(holder.binding.rvSubGroup.getVisibility() == View.GONE){
                holder.binding.rvSubGroup.setVisibility(View.VISIBLE);
                holder.binding.ivChevron.setImageDrawable(context.getDrawable(R.drawable.chevron_up));
            } else {
                holder.binding.rvSubGroup.setVisibility(View.GONE);
                holder.binding.ivChevron.setImageDrawable(context.getDrawable(R.drawable.chevron_down));
            }
        });
    }

    @Override
    public int getItemCount() {
        return resultPermits.size();
    }
}
